@@ -9,25 +9,39 @@ var __6MODEL_CORE__ = require('./bootstrap.js').core;
 var Hash = require('./hash.js');
 var Int64 = require('node-int64');
 var CodeRef = require('./code-ref.js');
+var constants = require('./constants.js');
+var NQPArray = require('./array.js');
+
+var containerSpecs = require('./container-specs.js');
+
+var hll = require('./hll.js');
 
 
 /** All the loaded serialization contexts using their unique IDs as keys */
-var serialization_contexts = SerializationContext.contexts;
+var serializationContexts = SerializationContext.contexts;
 
 module.exports.wval = function(handle, idx) {
-  return serialization_contexts[handle].root_objects[idx];
+  return serializationContexts[handle].rootObjects[idx];
 };
 
-op.deserialize = function(blob, sc, sh, code_refs, conflict) {
+op.deserialize = function(blob, sc, sh, codeRefs, conflict) {
   var buffer = new Buffer(blob, 'base64');
-  sc.code_refs = code_refs;
+  sc.codeRefs = codeRefs.array;
+
+  for (var i = 0; i < sc.codeRefs.length; i++) {
+    sc.codeRefs[i].isStatic = true;
+    sc.codeRefs[i]._SC = sc;
+    sc.rootCodes.push(sc.codeRefs[i]);
+  }
+
+  sh = sh.array;
   var cursor = new BinaryCursor(buffer, 0, sh, sc);
 
   cursor.deserialize(sc);
 };
 
 op.createsc = function(handle) {
-  return (serialization_contexts[handle] = new SerializationContext(handle));
+  return (serializationContexts[handle] = new SerializationContext(handle));
 };
 
 op.scsetdesc = function(sc, desc) {
@@ -44,7 +58,7 @@ op.scgethandle = function(sc) {
 };
 
 op.scobjcount = function(sc) {
-  return sc.root_objects.length;
+  return sc.rootObjects.length;
 };
 
 
@@ -101,7 +115,7 @@ BinaryCursor.prototype.array = function(readElem) {
   for (var i = 0; i < elems; i++) {
     array.push(readElem(this));
   }
-  return array;
+  return new NQPArray(array);
 };
 
 function StubedCodeRef(sc, id) {
@@ -129,54 +143,47 @@ BinaryCursor.prototype.objectEntry = function(objectsData) {
   var offset = this.I32();
 
   var sc = (packed >> OBJECTS_TABLE_ENTRY_SC_SHIFT) & OBJECTS_TABLE_ENTRY_SC_MASK;
-  var sc_idx;
+  var scIdx;
 
   if (sc == OBJECTS_TABLE_ENTRY_SC_OVERFLOW) {
     throw new Error('Objects Overflow NYI');
   } else {
-    sc_idx = packed & OBJECTS_TABLE_ENTRY_SC_IDX_MASK;
+    scIdx = packed & OBJECTS_TABLE_ENTRY_SC_IDX_MASK;
   }
 
   return {
-    STable: [sc, sc_idx],
+    STable: [sc, scIdx],
     data: this.at(objectsData + offset),
-    is_concrete: packed & OBJECTS_TABLE_ENTRY_IS_CONCRETE
+    isConcrete: packed & OBJECTS_TABLE_ENTRY_IS_CONCRETE
   };
 };
 
-BinaryCursor.prototype.locate_thing = function(thing_type) {
+BinaryCursor.prototype.locateThing = function(thingType) {
   var packed;
-  var sc_id;
+  var scId;
   var index;
   var sc;
 
-  const v15 = true;
-
   const PACKED_SC_SHIFT = 20;
   const PACKED_SC_OVERFLOW = 0xfff;
   const PACKED_SC_IDX_MASK = 0x000fffff;
 
-  if (v15) {
-    packed = this.U32();
-    sc_id = packed >>> PACKED_SC_SHIFT;
-    index = packed & PACKED_SC_IDX_MASK;
+  packed = this.varint();
+  scId = packed >>> PACKED_SC_SHIFT;
+  index = packed & PACKED_SC_IDX_MASK;
 
-    if (sc_id != PACKED_SC_OVERFLOW) {
-      index = packed & PACKED_SC_IDX_MASK;
-    } else {
-      sc_id = this.I32();
-      index = this.I32();
-    }
+  if (scId != PACKED_SC_OVERFLOW) {
+    index = packed & PACKED_SC_IDX_MASK;
   } else {
-    sc_id = this.I32();
-    index = this.I32();
+    scId = this.varint();
+    index = this.varint();
   }
 
   try {
-    return this.sc.deps[sc_id][thing_type][index];
+    return this.sc.deps[scId][thingType][index];
   } catch (e) {
     console.log(this.sc.deps.length);
-    console.log(e.toString(), sc_id, thing_type, index);
+    console.log(e.toString(), scId, thingType, index);
     throw e;
   }
 };
@@ -184,17 +191,17 @@ BinaryCursor.prototype.locate_thing = function(thing_type) {
 
 /**  */
 BinaryCursor.prototype.objRef = function() {
-  return this.locate_thing('root_objects');
+  return this.locateThing('rootObjects');
 };
 
 
 /** Read a hash of variants */
 BinaryCursor.prototype.hashOfVariants = function() {
-  var elems = this.I32();
+  var elems = this.varint();
   var hash = new Hash();
   for (var i = 0; i < elems; i++) {
     var str = this.str();
-    hash.content[str] = this.variant();
+    hash.content.set(str, this.variant());
   }
   return hash;
 };
@@ -234,16 +241,16 @@ BinaryCursor.prototype.varint = function() {
 
   result = (first & 0x0F) << 8 * need;
 
-  var shift_places = 0;
+  var shiftPlaces = 0;
   for (var i = 0; i < need; i++) {
     var byte = buffer.readUInt8(this.offset++);
-    result |= (byte << shift_places);
-    shift_places += 8;
+    result |= (byte << shiftPlaces);
+    shiftPlaces += 8;
   }
 
   if (need < 4) {
-      result = result << (64 - 4 - 8 * need);
-      result = result >> (64 - 4 - 8 * need);
+    result = result << (64 - 4 - 8 * need);
+    result = result >> (64 - 4 - 8 * need);
   }
 
   return result;
@@ -277,10 +284,10 @@ BinaryCursor.prototype.variant = function() {
       return this.hashOfVariants(this);
     case 11:
     case 12:
-      var codeRef = this.locate_thing('code_refs');
+      var codeRef = this.locateThing('codeRefs');
       if (!codeRef) {
         console.log('missing code ref while deserializing');
-        console.log(this.sc.code_refs);
+        console.log(this.sc.codeRefs);
       }
       return codeRef;
     default:
@@ -291,9 +298,11 @@ BinaryCursor.prototype.variant = function() {
 };
 
 var STABLE_BOOLIFICATION_SPEC_MODE_MASK = 0x0F;
-var STABLE_HAS_CONTAINER_SPEC           = 0x10;
-var STABLE_HAS_INVOCATION_SPEC          = 0x20;
-var STABLE_HAS_HLL_OWNER                = 0x40;
+var STABLE_HAS_CONTAINER_SPEC = 0x10;
+var STABLE_HAS_INVOCATION_SPEC = 0x20;
+var STABLE_HAS_HLL_OWNER = 0x40;
+var STABLE_HAS_HLL_ROLE = 0x80;
+
 
 /** Read an entry from the STable table */
 BinaryCursor.prototype.STable = function(STable) {
@@ -301,65 +310,103 @@ BinaryCursor.prototype.STable = function(STable) {
   STable.WHAT = this.objRef();
   STable.WHO = this.variant();
 
-  var method_cache = this.variant();
+  var methodCache = this.variant();
 
-  if (method_cache instanceof Hash) {
-    STable.setMethodCache(method_cache.content);
+  if (methodCache instanceof Hash) {
+    STable.setMethodCache(methodCache.$$toObject());
   }
 
-  var type_check_cache = [];
-  var type_check_cache_len = this.varint();
-  for (var i = 0; i < type_check_cache_len; i++) {
-    type_check_cache.push(this.variant());
+  var typeCheckCache = [];
+  var typeCheckCacheLen = this.varint();
+  for (var i = 0; i < typeCheckCacheLen; i++) {
+    typeCheckCache.push(this.variant());
   }
-  STable.type_check_cache = type_check_cache;
 
-  STable.mode_flags = this.U8();
+  if (typeCheckCache.length != 0) {
+    STable.typeCheckCache = typeCheckCache;
+  }
+
+  STable.modeFlags = this.U8();
 
   var flags = this.U8();
-  var boolification_mode = flags & 0xF;
+  var boolificationMode = flags & 0xF;
 
-  if (boolification_mode != 0xF) {
-    var boolification_method = this.variant();
-    STable.setboolspec(boolification_mode, boolification_method);
+  if (boolificationMode != 0xF) {
+    var boolificationMethod = this.variant();
+    STable.setboolspec(boolificationMode, boolificationMethod);
   }
 
   if (flags & STABLE_HAS_CONTAINER_SPEC) {
-    console.log("TODO container spec");
-    /* TODO - container stuff */
-   /* STable.container_class_handle = this.variant();
-    STable.container_attr_name = this.str();
-    STable.container_hint = this.I64();
-    STable.container_fetch_method = this.variant();*/
+    var specType = this.str();
+    STable.containerSpec = new containerSpecs[specType](STable);
+    STable.containerSpec.deserialize(this);
   }
 
   if (flags & STABLE_HAS_INVOCATION_SPEC) {
-    /* TODO */ 
-    console.log("TODO invocation spec");
-    /*
-    STable.invocation_class_handle = this.variant();
-    STable.invocation_attr_name = this.str();
-    STable.invocation_hint = this.I64();
-    STable.invocation_handler = this.variant();
-    */
+    var classHandle = this.variant();
+    var attrName = this.str();
+    this.varint(); // hint
+    var invocationHandler = this.variant();
+
+    this.variant(); // md_class_handle
+    this.str(); // md_cache_attr_name
+    this.varint(); // md_cache_hint
+    this.str(); // md_valid_attr_name
+    this.varint(); // md_valid_hint
+
   }
 
   if (flags & STABLE_HAS_HLL_OWNER) {
-    STable.hll_owner = this.str();
+    STable.hllOwner = hll.getHLL(this.str());
   }
 
-  // TODO check for MVM_PARAMETRIC_TYPE mode_flags
-  // TODO check for MVM_PARAMETRIC_TYPE mode_flags
+  if (flags & STABLE_HAS_HLL_ROLE) {
+    STable.hllRole = this.varint();
+  }
 
-  if (STable.REPR.deserialize_repr_data) {
-    STable.REPR.deserialize_repr_data(this.clone(), STable);
+  if (STable.modeFlags & constants.PARAMETRIC_TYPE) {
+    STable.parameterizer = this.variant();
+
+    if (!STable.parameterizerCache) {
+      STable.parameterizerCache = [];
+    }
+
+  }
+
+  if (STable.modeFlags & constants.PARAMETERIZED_TYPE) {
+    STable.parametricType = this.variant();
+    var count = this.varint();
+    var params = [];
+    for (var i = 0; i < count; i++) {
+      params[i] = this.variant();
+    }
+
+    var ptable = STable.parametricType._STable;
+
+    if (!ptable.parameterizerCache) {
+      ptable.parameterizerCache = [];
+    }
+
+    STable.parameters = new NQPArray(params);
+
+    ptable.parameterizerCache.push({type: STable.WHAT, params: params});
+  }
+
+  STable.debugName = this.cstr();
+
+  if (STable.REPR.deserializeReprData) {
+    STable.REPR.deserializeReprData(this.clone(), STable);
+  }
+
+  if (flags & STABLE_HAS_INVOCATION_SPEC) {
+    STable.setinvokespec(classHandle, attrName, invocationHandler);
   }
 
 
 };
 
 BinaryCursor.prototype.staticCodeRef = function() {
-  var staticCode = this.locate_thing('code_refs');
+  var staticCode = this.locateThing('codeRefs');
   if (!staticCode) {
     console.log('Code ref has an invalid static code');
   }
@@ -370,15 +417,13 @@ BinaryCursor.prototype.closureEntry = function() {
   var entry = {};
   var staticScId = this.I32();
   var staticIndex = this.I32();
-  entry.staticCode = this.sc.deps[staticScId].code_refs[staticIndex];
-  //entry.staticCode = this.staticCodeRef();
+  entry.staticCode = this.sc.deps[staticScId].codeRefs[staticIndex];
   entry.context = this.I32();
   var hasCodeObj = this.I32();
   if (hasCodeObj) {
-    //entry.codeObj = this.objRef();
     var objectScId = this.I32();
     var objectIndex = this.I32();
-    entry.codeObj = this.sc.deps[objectScId].root_objects[objectIndex];
+    entry.codeObj = this.sc.deps[objectScId].rootObjects[objectIndex];
   } else {
     // we're packed along a 24-byte alignment
     this.I32();
@@ -391,14 +436,13 @@ BinaryCursor.prototype.contextEntry = function(contextsData) {
   var entry = {};
   var staticScId = this.I32();
   var staticIndex = this.I32();
-  entry.staticCode = this.sc.deps[staticScId].code_refs[staticIndex];
-  //entry.staticCode = this.staticCodeRef();
+  entry.staticCode = this.sc.deps[staticScId].codeRefs[staticIndex];
   var data = this.at(contextsData + this.I32());
   entry.outer = this.I32();
   entry.inner = [];
   entry.closures = [];
 
-  var count = data.I64();
+  var count = data.varint();
   var info = entry.staticCode.staticInfo;
 
   var lexicals = {};
@@ -415,7 +459,7 @@ BinaryCursor.prototype.contextEntry = function(contextsData) {
         lexicals[name] = data.variant();
         break;
       case 1: // int
-        lexicals[name] = data.I64();
+        lexicals[name] = data.varint();
         break;
       case 2: // num
         lexicals[name] = data.double();
@@ -437,24 +481,24 @@ BinaryCursor.prototype.deserialize = function(sc) {
 
   this.sc = sc;
 
-  if (version != 16) {
+  if (version != 20) {
     throw 'Unsupported serialization format version: ' + version;
   }
 
-  var deps_offset = this.I32();
-  var deps_number = this.I32();
+  var depsOffset = this.I32();
+  var depsNumber = this.I32();
 
 
-  var dependencies = this.at(deps_offset).times(deps_number,
+  var dependencies = this.at(depsOffset).times(depsNumber,
       function(cursor) { return [cursor.str32(), cursor.str32()]; });
 
   var deps = [sc];
   this.sc.deps = deps;
 
   for (var i in dependencies) {
-    var dep = serialization_contexts[dependencies[i][0]];
+    var dep = serializationContexts[dependencies[i][0]];
     if (!dep) {
-      //console.log(Object.keys(serialization_contexts));
+      //console.log(Object.keys(serializationContexts));
       console.log(
           "Missing dependencie, can't find serialization context handle:",
           dependencies[i][0],
@@ -465,21 +509,21 @@ BinaryCursor.prototype.deserialize = function(sc) {
   }
 
 
-  var STables_offset = this.I32();
-  var STables_number = this.I32();
-  var STables_data = this.I32();
+  var STablesOffset = this.I32();
+  var STablesNumber = this.I32();
+  var STablesData = this.I32();
 
-  var STables = this.at(STables_offset).times(STables_number,
+  var STables = this.at(STablesOffset).times(STablesNumber,
       function(cursor) {
-        return [cursor.str32(), cursor.at(STables_data + cursor.I32()), cursor.at(STables_data + cursor.I32())];
+        return [cursor.str32(), cursor.at(STablesData + cursor.I32()), cursor.at(STablesData + cursor.I32())];
       });
 
 
-  var objects_offset = this.I32();
-  var objects_number = this.I32();
-  var objects_data = this.I32();
-  var objects = this.at(objects_offset).times(objects_number, function(cursor) {
-    return cursor.objectEntry(objects_data);
+  var objectsOffset = this.I32();
+  var objectsNumber = this.I32();
+  var objectsData = this.I32();
+  var objects = this.at(objectsOffset).times(objectsNumber, function(cursor) {
+    return cursor.objectEntry(objectsData);
   });
 
 
@@ -493,86 +537,94 @@ BinaryCursor.prototype.deserialize = function(sc) {
     var REPR = new reprs[repr]();
     REPR.name = repr;
     var HOW = null; /* We will fill that in later once objects are stubbed */
-    sc.root_stables[i] = new sixmodel.STable(REPR, HOW);
-    sc.root_stables[i]._SC = sc;
+    sc.rootStables[i] = new sixmodel.STable(REPR, HOW);
+    sc.rootStables[i]._SC = sc;
   }
 
   /* Stub objects */
   for (var i = 0; i < objects.length; i++) {
-    var STable_for_obj =
-        deps[objects[i].STable[0]].root_stables[objects[i].STable[1]];
-    if (!STable_for_obj) {
-      console.log('Missing stable', objects[i].STable[0], objects[i].STable[1], deps[objects[i].STable[0]].root_stables);
+    var STableForObj =
+        deps[objects[i].STable[0]].rootStables[objects[i].STable[1]];
+    if (!STableForObj) {
+      console.log('Missing stable', objects[i].STable[0], objects[i].STable[1], deps[objects[i].STable[0]].rootStables);
     }
 
-    objects[i].is_array = objects[i].is_concrete && STable_for_obj.REPR.name == 'VMArray';
-    objects[i].array_repr = STable_for_obj.REPR;
+    objects[i].isArray = objects[i].isConcrete && STableForObj.REPR.name == 'VMArray';
+    objects[i].arrayRepr = STableForObj.REPR;
 
-    if (objects[i].is_array) {
-      sc.root_objects[i] = [];
+    if (objects[i].isArray) {
+      sc.rootObjects[i] = new NQPArray([]);
     } else {
-      sc.root_objects[i] = objects[i].is_concrete
-        ? new (STable_for_obj.obj_constructor)()
-        : STable_for_obj.createTypeObject();
-      sc.root_objects[i]._SC = sc;
+      sc.rootObjects[i] = objects[i].isConcrete ?
+          new (STableForObj.objConstructor)() :
+          STableForObj.createTypeObject();
+      sc.rootObjects[i]._SC = sc;
     }
   }
 
-  for (var i = 0; i < STables.length; i++) {
-    STables[i][1].STable(sc.root_stables[i]);
-  }
 
-  var closures_offset = this.I32();
-  var closures_number = this.I32();
-  var closures = this.at(closures_offset).times(closures_number, function(cursor) {
+  var closuresOffset = this.I32();
+  var closuresNumber = this.I32();
+  var closures = this.at(closuresOffset).times(closuresNumber, function(cursor) {
     return cursor.closureEntry();
   });
 
-  var closures_base = sc.code_refs.length;
+  var closuresBase = sc.codeRefs.length;
   for (var i = 0; i < closures.length; i++) {
-    sc.code_refs[closures_base + i] = new CodeRef(closures[i].staticCode.name);
-    if (closures[i].codeObj) sc.code_refs[closures_base + i].codeObj = closures[i].codeObj;
-    closures[i].index = closures_base + i;
+    sc.codeRefs[closuresBase + i] = new CodeRef(closures[i].staticCode.name);
+    if (closures[i].codeObj) sc.codeRefs[closuresBase + i].codeObj = closures[i].codeObj;
+    closures[i].index = closuresBase + i;
+
+    closures[i]._SC = sc;
+    sc.rootCodes.push(closures[i]);
+  }
+
+  for (var i = 0; i < STables.length; i++) {
+    STables[i][1].STable(sc.rootStables[i]);
   }
 
   /* Finish up objects */
   for (var i = 0; i < objects.length; i++) {
-    if (objects[i].is_array) {
-      var repr = objects[i].array_repr;
-      repr.deserialize_array(sc.root_objects[i], objects[i].data);
-    } else if (objects[i].is_concrete) {
-      var repr = sc.root_objects[i]._STable.REPR;
-      if (repr.deserialize_finish) {
-        repr.deserialize_finish(sc.root_objects[i], objects[i].data);
+    if (objects[i].isArray) {
+      var repr = objects[i].arrayRepr;
+      repr.deserializeArray(sc.rootObjects[i], objects[i].data);
+    } else if (objects[i].isConcrete) {
+      var repr = sc.rootObjects[i]._STable.REPR;
+      if (repr.deserializeFinish) {
+        repr.deserializeFinish(sc.rootObjects[i], objects[i].data);
       }
     }
   }
 
-  var contexts_offset = this.I32();
-  var contexts_number = this.I32();
-  var contexts_data = this.I32();
-  var contexts = this.at(contexts_offset).times(contexts_number, function(cursor) {
-    return cursor.contextEntry(contexts_data);
+  var contextsOffset = this.I32();
+  var contextsNumber = this.I32();
+  var contextsData = this.I32();
+  var contexts = this.at(contextsOffset).times(contextsNumber, function(cursor) {
+    return cursor.contextEntry(contextsData);
   });
 
   for (var i = 0; i < contexts.length; i++) {
     if (contexts[i].outer) contexts[contexts[i].outer - 1].inner.push(contexts[i]);
   }
 
-  var no_context_closures = [];
+  var noContextClosures = [];
 
 
   for (var i = 0; i < closures.length; i++) {
     if (closures[i].context) {
       contexts[closures[i].context - 1].closures.push(closures[i]);
     } else {
-      no_context_closures.push(closures[i]);
+      noContextClosures.push(closures[i]);
     }
   }
 
-  var code = no_context_closures.map(function(closure) {
-    return 'var ' + closure.staticCode.outerCtx + ' = null;\n' +
-        'sc.code_refs[' + closure.index + '].block(' +
+  var code = noContextClosures.map(function(closure) {
+    var type = closure.staticCode.asMethod ? 'method' : 'block';
+    var codeRef = 'sc.codeRefs[' + closure.index + ']';
+
+    return 'var ' + closure.staticCode.outerCtxVar + ' = null;\n' +
+        'var $$codeRef = ' + codeRef + ';\n' +
+        'sc.codeRefs[' + closure.index + '].' + type + '(' +
         closure.staticCode.closureTemplate +
         ');\n';
   }).join('');
@@ -586,16 +638,17 @@ BinaryCursor.prototype.deserialize = function(sc) {
 
   var cuids = [];
   for (var cuid in CodeRef.cuids) {
-    mangledCuid = cuid.replace(/\./g, '_');
+    mangledCuid = 'cuid' + cuid;
     cuids.push(mangledCuid + ' = CodeRef.cuids[\"' + cuid + '\"]');
   }
 
   var declareCuids = 'var ' + cuids.join(',') + ';\n';
 
-  var prelude = "var nqp = require('nqp-runtime');\n";
+  var prelude = ';\n';
   if (code) {
     /* TODO reduce accidental poisoning */
     /* TODO make cuids be in scope */
+    var nqp = require('nqp-runtime');
     eval(prelude + declareCuids + code);
   }
 
@@ -610,25 +663,38 @@ BinaryCursor.prototype.deserialize = function(sc) {
 };
 
 BinaryCursor.prototype.contextToCode = function(context, data) {
-  var outer_ctx = 'null'; // TODO
-  var caller_ctx = 'null';
-  var create_ctx = 'var ' + context.staticCode.ctx + ' = new nqp.Ctx(' + outer_ctx + ', ' + caller_ctx + ');\n';
-  var set_vars = '';
+  var outerCtx = 'null'; // TODO
+  var callerCtx = 'null';
+  var createCtx = 'var ' + context.staticCode.ctx + ' = new nqp.Ctx(' + outerCtx + ', ' + callerCtx + ');\n';
+  var setVars = '';
 
 
   var lexicals = [];
+  function addToData(value) {
+    data.push(value);
+    return 'data[' + (data.length - 1) + ']';
+  }
+
   for (var name in context.lexicals) {
-    data.push(context.lexicals[name]);
-    set_vars += 'var ' + context.staticCode.staticInfo[name][1] + ' = data[' + (data.length - 1) + ']\n';
-    console.l;
+    var value = context.lexicals[name];
+    var info = context.staticCode.staticInfo[name];
+
+    var getData = 'data[' + (data.length - 1) + ']\n';
+    if (info.length == 2) {
+      setVars += ('var ' + info[1] + ' = ' + addToData(value) + '\n');
+    } else {
+      setVars += (context.staticCode.ctx + '[' + addToData(name) + '] = ' + addToData(value) + '\n');
+    }
   }
 
   return '(function() {\n' +
-      create_ctx +
-      set_vars +
+      createCtx +
+      setVars +
       context.inner.map(function(inner) {return this.contextToCode(inner, data)}).join('') +
       context.closures.map(function(closure) {
-        return 'sc.code_refs[' + closure.index + '].block(' +
+        var type = closure.staticCode.asMethod ? 'method' : 'block';
+        var codeRef = 'sc.codeRefs[' + closure.index + ']';
+        return 'var $$codeRef = ' + codeRef + ';\n' + codeRef + '.' + type + '(' +
            closure.staticCode.closureTemplate +
            ');\n';
       }).join('') +
@@ -707,15 +773,16 @@ BinaryCursor.prototype.flag64 = function() {
 };
 
 var STRING_HEAP_LOC_PACKED_OVERFLOW = 0x00008000;
-var STRING_HEAP_LOC_PACKED_SHIFT    = 16;
+var STRING_HEAP_LOC_PACKED_SHIFT = 16;
+
 
 /** Read a String */
 BinaryCursor.prototype.str = function() {
   var offset = this.U16();
   if (offset & STRING_HEAP_LOC_PACKED_OVERFLOW) {
-      offset ^= STRING_HEAP_LOC_PACKED_OVERFLOW;
-      offset <<= STRING_HEAP_LOC_PACKED_SHIFT;
-      offset |= this.U16();
+    offset ^= STRING_HEAP_LOC_PACKED_OVERFLOW;
+    offset <<= STRING_HEAP_LOC_PACKED_SHIFT;
+    offset |= this.U16();
   }
   if (!this.sh.hasOwnProperty(offset)) {
     throw "can't read str: " + offset;
@@ -723,6 +790,15 @@ BinaryCursor.prototype.str = function() {
   return this.sh[offset];
 };
 
+
+/** Read a C String */
+BinaryCursor.prototype.cstr = function() {
+  var len = this.varint();
+  var str = this.buffer.slice(this.offset, len).toString('utf8');
+  this.offset += len;
+  return str;
+};
+
 BinaryCursor.prototype.str32 = function() {
   return this.sh[this.I32()];
 };
