@@ -155,6 +155,10 @@ public final class NativeCallOps {
                         case DOUBLE_RW:
                             ((NativeRefInstance) o).store_n(tc, ((Memory) cArgs[i]).getDouble(0));
                             break;
+                        case CPOINTER_RW:
+                            o = Ops.decont(o, tc);
+                            ((CPointerInstance) o).set_int(tc, Pointer.nativeValue(((Memory) cArgs[i]).getPointer(0)));
+                            break;
                         default:
                             refresh(o, tc);
                             break;
@@ -600,6 +604,16 @@ public final class NativeCallOps {
             m.setDouble(0, (double) ((NativeRefInstance) o).fetch_n(tc));
             return m;
         }
+        case CPOINTER_RW: {
+            Memory m = new Memory(Pointer.SIZE);
+            o        = Ops.decont(o, tc);
+            long ptr = ((CPointerInstance) o).get_int(tc);
+            if (ptr > 0)
+                m.setPointer(0, Pointer.createConstant(ptr));
+            else
+                m.setPointer(0, null);
+            return m;
+        }
         default:
             throw ExceptionHandling.dieInternal(tc, String.format("Don't know how to convert %s arguments to JNA yet", target));
         }
